# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python

from typing import Any, Optional

def unescaped_str(arg_str: Any): ...

PARSER: Any

def main(ARGS: Optional[Any] = ...) -> None: ...
